#!/bin/bash

# Makefile to compile up the OWASP-for-C package after first compiling up its dependencies
# The OWASP-for-C package:
# http://code.google.com/p/owasp-esapi-c/
# http://owasp-esapi-c.googlecode.com/svn/trunk/doc/html/index.html
# 3 of its Makefiles were changed to get things compiling: top level, src and example folders

# unpack everything
# (svn checkout owasp-for-c? or commit as a directory to gs?)
# put uthash/src/* into owasp-for-c/include/.
# create 'installed' folder
# compile everything (or at least all the dependencies) into the installed folder, compile owasp

securitydir=`pwd`
installdir=$securitydir/installed

LIBGPGERROR=libgpg-error-1.12
LIBGCRYPT=libgcrypt-1.6.1
LOG4C=log4c-1.2.4
UTHASH=uthash-master
OWASPC=gs_owasp-esapi-c_27.02.2014
# svn checkout http://owasp-esapi-c.googlecode.com/svn/trunk/ owasp-esapi-c-read-only


# If doing tarclean: then get rid of all the extracted folders and the installed folder
if [ "x$1" = "xtarclean" ]; then
    folders="$LIBGPGERROR $LIBGCRYPT $LOG4C $UTHASH $OWASPC $installdir"
    for folder in $folders; do
	if [ -d $folder ]; then
	    rm -rf $folder
	fi
    done
    exit
fi


# make the installed folder
if [ ! -e $installdir ]; then
    mkdir $installdir
fi


# UNPACK
bziplist="$LIBGPGERROR $LIBGCRYPT $OWASPC $UTHASH"
for bzip in $bziplist; do
    if test ! -d "$bzip" ; then
	tar xvjf $bzip.tar.bz2
    fi
done
if test ! -d "$LOG4C" ; then
    tar xvzf $LOG4C.tar.gz
fi
#cp $UTHASH/src/* $OWASPC/include/.


# If doing a make clean or distclean
pkglist=
if [ "x$1" != "x" ]; then
    pkglist="$LIBGPGERROR $LIBGCRYPT $OWASPC $LOG4C"
    for d in $pkglist; do
	cd $d
	make $1
	cd $securitydir
    done
    exit 0
fi

# COMPILE DEPENDENCIES
# The dependencies of OWASP are LIBGCRYPT, LOG4C and UTHASH. 
# The last is just header files that gets copied over into the src folder
# LIBGCRYPT is dependent on LIBGPGERROR, so that needs to be compiled up first

pkglist="$LIBGPGERROR $LIBGCRYPT $LOG4C"
for d in $pkglist; do
    echo "    Compiling $d"

    cd $d
    

    # configure, but only if we haven't already
    if [ ! -e Makefile ] &&  [ ! -e config.h ] ; then

        # if we're making static libraries (.a) files in the lib folder, add --disable-shared to the configure step
        # if we're making .so (shared object files/dlls) in the lib folder, then remove --disable-shared from configure

	if [ "$d" = "$LIBGCRYPT" ]; then
	    ./configure --with-gpg-error-prefix=$installdir --prefix=$installdir --disable-shared
	else
	    ./configure --prefix=$installdir --disable-shared
	fi
    fi

    if [ $? != 0 ] ; then
	echo "Error encountered when configuring $d"
	exit 1
    fi

    # make
    make
    if [ $? != 0 ] ; then
	echo "Error encountered making $d"
	exit 1
    fi

    # make install
    make install
    if [ $? != 0 ] ; then
	echo "Error encountered when installing $d"
	exit 1
    fi

    cd $securitydir
done

# COMPILE OWASP ESAPI for C
echo "    Compiling $OWASPC"

cd $OWASPC/
CFLAGS=-I$installdir/include LDFLAGS=-L$installdir/lib prefix=$installdir make
CFLAGS=-I$installdir/include LDFLAGS=-L$installdir/lib prefix=$installdir make install
#CFLAGS=-I$installdir/include LDFLAGS=-L$installdir/lib prefix=$installdir make examples
cd $securitydir

# To get the OWASP example working, need to compile manually in the example folder,
# since the above 'make examples' command does not work

# make examples does the following (all on one line):

#/.../security/owasp-esapi-c_27.02.2014/example>cc -I$installdir/include 
#-O0 -g3 -ggdb -DDEBUG=1 -std=c99 -Wall -Wextra -Wno-unused -Wformat=2 -Wformat-security -fstack-protector-all -pedantic -c -static 
#-I../include -I../../include access_ref_example.c -o access_ref_example -L../lib -L$installdir/lib 
#-Wl,-z,nodlopen -Wl,-z,nodldump -Wl,-z,relro -Wl,-z,now 
#-lesapi -lgcrypt -lgpg-error -llog4c 

# while the following works (run all on one line):

#/.../security/owasp-esapi-c_27.02.2014/example>cc -I$installdir/include 
#-I../include -I../../include access_ref_example.c -o access_ref_example -L../lib -L$installdir/lib  
#-lesapi -lgcrypt -lgpg-error -llog4c